import unittest
import sys
sys.path.append('..')
from selenium import webdriver
from selenium.webdriver.common.keys import Keys
import time
import pandas as pd

def start_driver():
    """
    Starts a webdriver driver using the chromedriver in the folder. 

    returns driver
    """
    driver = webdriver.Chrome('chromedriver.exe') 
    return driver
#start_driver()
#This was clearly a success



def reset_variable(input):
    """
    Depending on the type of input. It will give some default values.
    
    can allow string integer list and boolean.
    returns same type of variable as input. 
    """
    
    
    if type(input) == str:
        input = ''
    if type(input) == int:
        input = 0
    if type(input) == list:
        input = []
    if type(input) == bool:
        """
        Be weary. This will always default to false (There are more false things than true things initally).
        """
        input = False
    return input

def test_reset_variable():
    """
    Checking if the main type of inputs are reset
    """
    test_list = []
    expectted_value = ['', 0, [], False]
    test_list.append(reset_variable('h')) #This is invisible 
    test_list.append(reset_variable(13))
    test_list.append(reset_variable([12, '']))
    test_list.append(reset_variable(True))
    print(expectted_value == test_list)

#test_reset_variable()





def create_dict(*args):
    """
    Creates a dictionary with argument names as key names and values as values.

    input valued variables. 
    output dictionary
    """
    return dict({i:eval(i) for i in args})


a = 15
b = 20
def test_create_dict():

    print({'a': 15, 'b': 20} == create_dict('a', 'b'))

#test_create_dict()


def seperate_month_and_year(month_and_year):
    """
    month_and_year is normally found as a size 2 tuple. This seperates it into components.

    input length 2 tuple. output length two list.

    TD have this raise a value error if something other than the usual input is inserted. 
    """
    year, month = month_and_year
    dummy_list = [year, month]
    return dummy_list

def test_seperate_month_and_year():
    expected_value = [2020, 'May']
    print(expected_value == seperate_month_and_year((2020, 'May')))
test_seperate_month_and_year()
#TD test a large scope of more actual months and years. 









#print(test_seperate_month_and_year((2020, 'May')))





from test_main_classes import Parameters_selection, Rigid_Parameters_Selection
import unittest
from selenium import webdriver
from selenium.webdriver.common.keys import Keys
from test_main_classes import Scraping_Space
import time
import pandas as pd
from Project.common_functions import *
import sys
sys.path.append('..')
our_parameters = Parameters_selection
rigid_parameters = Rigid_Parameters_Selection

class Wikipedia_Xpath_selection():
    """
    This is where the user can change certain xpaths easily without having to find all the usages in the code. 

    These cover the xpaths from wikipeeida. The function title normally contains the name of the usual variable name making it easy to type in and refer to. 
    These all return strings 

    TD: Make this a subclass of some sort of all encompassing variable selector. 

    """
    def get_five_hundred_button():
        """
        This is the button that changes the amount of edit histories displayed from 50 to 500, a better page to scrape from
        """
        return '/html/body/div[3]/div[3]/div[4]/a[7]'
        
    def get_full_edit_list():
        return '//*[@id="pagehistory"]'
    def get_table_header():
        return '//*[@id="mw-content-text"]/div[1]/table[2]/tbody/tr[1]'
    def get_page_table_unchanged_format():
        """
        At some point the format of the list changes. Different xpaths are needed depending on this. 
        """
        return '/html/body/div[3]/div[3]/div[5]/div[1]/table[2]/tbody'
    def get_page_table_changed_format():
        """
        At some point the format of the list changes. Different xpaths are needed depending on this. 
        """
        return '/html/body/div[3]/div[3]/div[5]/div[1]/table[1]/tbody'
wikipedia_xpaths = Wikipedia_Xpath_selection

def test_wikipedia_xpath_selection():
    expected_value = '/html/body/div[3]/div[3]/div[4]/a[7]'
    output = wikipedia_xpaths.get_five_hundred_button()
    return expected_value == output

#print(test_wikipedia_xpath_selection())





class Stock_Scraping_Space(Scraping_Space):
    """
    This is the scraping mechnism for the stock exchange, the wikipedia pages, specifically. 
    """
    def __init__(self, sleeping_amounts = []):
        """
        We are inheriting from the Scraping Space. 
        """
        super().__init__()
        self.sleeping_amounts = sleeping_amounts

    @staticmethod
    def go_to_home(exchange_URL, driver):
        """
        This static method takes us to our homepage.
        """
        driver.get(exchange_URL)
        fivehundred_link = driver.find_element_by_xpath(wikipedia_xpaths.get_five_hundred_button())
        fivehundred_link.click()


   # def check_format_change(change_of_format, second_driver):
    #        if  change_of_format == False:
     #           page_table = second_driver.find_element_by_xpath('/html/body/div[3]/div[3]/div[5]/div[1]/table[2]/tbody')
      #      else:
       #         page_table = second_driver.find_element_by_xpath('/html/body/div[3]/div[3]/div[5]/div[1]/table[1]/tbody')
    @staticmethod
    def check_format_change(written_date_text):
        if written_date_text in our_parameters.return_format_change_dates():
            return True
        else:
            return False

    @staticmethod
    def check_end_date(written_date_text):
        if written_date_text in our_parameters.return_end_date():
            return True
        else:
            return False
    



    def return_edit_list(self, driver):
        """
        This method gives us the list of edits to go through from the home page. This would work if and only if we are at the homepage. 
        """
        try:
            return self.get_all_children(driver, wikipedia_xpaths.get_full_edit_list())
        except:
            pass
    
    @staticmethod
    def return_page_table(second_driver, change_of_format):
        if change_of_format == False:
            return second_driver.find_element_by_xpath(wikipedia_xpaths.get_page_table_unchanged_format())
        else:
            return second_driver.find_element_by_xpath(wikipedia_xpaths.get_page_table_changed_format())

    def complete_stock_scrape(self):
        """
        This is the complete process of scraping for data from wikipedia. 
        
        """
        completed_months_and_years = []
        complete_list = []
        list_of_locations = []
        change_of_format = False
        driver = start_driver()
        self.go_to_home(our_parameters.return_exchange_URL(), driver)
        edit_list = self.return_edit_list(driver)
        self.return_edit_list(edit_list, driver, complete_list, list_of_locations, completed_months_and_years)


    @staticmethod
    def get_month_and_year_from_text(written_date_text = '12:07, 27 August 2021'):
        """
        This returns the month and year given a string containing both.

        input: String

        Output: tuple of year and month
        """
        current_year =  written_date_text[-4:]
        for month in rigid_parameters.return_list_of_month_names():
            if month in written_date_text:
                current_month = month
        return (current_year, current_month)
    
    @staticmethod
    def scrape_through_rows_of_table(list_of_rows, this_months_dictionary):
        """
        We go to a table and can scrapped through it to get imformation. There are no nested scraping methods inside this method. 
        """
        
        for row in list_of_rows:
            list_of_row_elements = row.find_elements_by_xpath("*")
            for row_elements in list_of_row_elements:
                k = 1   #This k indicates which column we are concerned with

                if len(list_of_row_elements) == 16:
                    if k == 1:  #This column corresponds to the rank of the exchange. 
                        this_months_dictionary['months_rank_list'].append(row_elements.text)

                    if k ==3: #This column corresponds to the name of the exchange.
                        this_months_dictionary['months_exchange_list'].append(row_elements.text)

                    if k == 6:
                        split_locations = row_elements.text.split('\n')
                        this_months_dictionary['months_location_list'].append(split_locations)

                    if k == 7:  #This column corresponds to the market's market cap. 
                        this_months_dictionary['months_market_cap_list'].append(row_elements.text)

                    if k ==8: # This column corresponds to the market's trade volume
                        this_months_dictionary['months_trade_volume_list'].append(row_elements.text)
            



                k += 1 #  Move to the next column 
            return(this_months_dictionary)

    def scrape_current_month(self,  change_of_format, written_date_link, this_months_dictionary = {'months_rank_list' : [], 'months_exchange_list' : [], 'months_location_list' : [], 'months_market_cap_list' : [], 'months_trade_volume_list' : [], 'current_column' : [], 'table_header_children' : []}):
        """
        A month has been selected. We can scrapped through the wikipedia page for this month. 
        """
        global months_rank_list, months_exchange_list, months_location_list, months_market_cap_list, months_trade_volume_list, current_column, table_header_children
        months_exchange_list = []
        months_location_list = []
        months_market_cap_list = []
        months_trade_volume_list = []
        #this_months_dictionary = create_dict("months_rank_list", "months_exchange_list", "months_location_list", "months_market_cap_list", "months_trade_volume_list")

        current_column = 0
                 
        second_driver = webdriver.Chrome('chromedriver.exe') # It makes it easier if we use a second driver to open links to a new window. 
        second_driver.get(written_date_link)
        

        table_header_children = self.get_all_children(second_driver, wikipedia_xpaths.get_table_header())
        page_table = self.return_page_table(second_driver, change_of_format)
        list_of_rows = page_table.find_elements_by_xpath("*")
        print(list_of_rows)
        this_months_dictionary = self.scrape_through_rows_of_table(list_of_rows, this_months_dictionary)


        second_driver.quit()
        return this_months_dictionary


    #@staticmethod
    #def rename_keys(this_months_dictionary):
        ##"""
        #It is convient to change the keys as they are less specific going forward.
        #"""
        ##this_months_dictionary['Rank'] = this_months_dictionary.pop('months_rank_list')
        #this_months_dictionary['Name of exchange'] = this_months_dictionary.pop('months_exchange_list')
        #this_months_dictionary['Locations'] = this_months_dictionary.pop('months_location_list')
        #this_months_dictionary['Market Cap'] = this_months_dictionary.pop('months_market_cap_list')
        #this_months_dictionary['Trade Voume'] = this_months_dictionary.pop('months_trade_volume_list')

    @staticmethod
    def add_locations_to_distinct_list(list_of_locations, this_months_dictionary):
        for location in this_months_dictionary['months_location_list']:
            if location not in list_of_locations:
                list_of_locations.append(location)

    @staticmethod
    def create_month_location_match(month_and_year, month_location_list):
        """
        This creats a dictionary that matches the months with the locations. 
        """
        dictonary = {}
        dictonary[month_and_year] = month_location_list
        return dictonary

    @staticmethod
    def return_last_scraped_month_data(complete_list, last_scraped_month):
        """
        This returns the data from the last scrapped month so we can copy it to the skipped month.
        """
        for month in complete_list:
            if month[0] == last_scraped_month:
                return month[1]

    def scrape_through_edit_list(self, edit_list, complete_list, list_of_locations, completed_months_and_years):
        """
        We are scraping through the edit list to get links and data from it. 
        """
        month_and_year = []
        
        last_scraped_month = ''
        for link in edit_list:
            reset_variable(month_and_year)
            written_date = link.find_element_by_class_name(our_parameters.return_date_text_identifier()) # The text here gives us the date
            written_date_link = written_date.get_attribute('href')
            written_date_text = written_date.text


            if self.skip_date(written_date_text) == True:
                continue
            
            month_and_year = self.get_month_and_year_from_text(written_date_text)
            
            
            if self.check_shall_month_be_scraped(month_and_year, completed_months_and_years) == True:
                this_months_dictionary = self.scrape_current_month(self.check_format_change(written_date_text), written_date_link)
                complete_list.append([month_and_year, this_months_dictionary])     
                print(complete_list)                   
                completed_months_and_years.append(month_and_year)
                self.add_locations_to_distinct_list(list_of_locations, this_months_dictionary)
                month_location_match = self.create_month_location_match(month_and_year, this_months_dictionary['months_location_list'])

                self.copy_data_to_skipped_month(complete_list, this_months_dictionary, completed_months_and_years, list_of_locations, month_and_year)

                last_scraped_month = month_and_year


            if self.check_end_date(written_date_text) == True:
                self.copy_data_to_skipped_month(complete_list,completed_months_and_years, list_of_locations, month_and_year, this_months_dictionary = self.return_last_scraped_month_data(complete_list, last_scraped_month))
                break
            
        return (complete_list, list_of_locations, month_location_match)

    
    def copy_data_to_skipped_month(self, complete_list, completed_months_and_years, list_of_locations, month_and_year, this_months_dictionary):           
        """
        If a desired month is skipped. The data fromt the next desired month is used as a subsitiute.

        A month can be skipped if there was no edit data for that month

        output: a list containgin data for that month. 
        """
        for skipped_month in our_parameters.return_desired_scraped_months():
            try:
                #This is succesfu; only if month and year is desired
                if our_parameters.return_desired_scraped_months().index(skipped_month) < our_parameters.return_desired_scraped_months().index(month_and_year) and skipped_month not in completed_months_and_years:
                    complete_list.append([skipped_month, this_months_dictionary])                        
                    completed_months_and_years.append(skipped_month)
                    self.add_locations_to_distinct_list(list_of_locations, this_months_dictionary)
                    self.create_month_location_match(month_and_year = skipped_month, location_list = this_months_dictionary['months_location_list'])       
            except:
                pass

    @staticmethod
    def check_if_month_is_desired(month_and_year):
        """
        This checks the desired month list to see if the inout is inside it.

        input: tuple: year and month

        output: True/False
        """
        if month_and_year in our_parameters.return_desired_scraped_months():
            
            return True

        else:
            return False
            


    def check_shall_month_be_scraped(self, month_and_year, completed_months_and_years):
        """

        """
        if month_and_year in completed_months_and_years:
            return False
        else:
            return self.check_if_month_is_desired(month_and_year)


    @staticmethod
    def find_next_desired_month(month_and_year):
        """
        We have a list of desired months. This method gives us the next month in that list given an inputed month in the list.

        input: tuple: year and month

        output: tuple: year and month
        """
        try:
            index = our_parameters.return_desired_scraped_months().index(month_and_year)
            return our_parameters.return_desired_scraped_months()[index + 1]
        except:  #This happens when we have reached the endff of our desired range. 
            return our_parameters.return_desired_scraped_months()

                
    @staticmethod
    def skip_date(written_date_text):
        """
        When a date is entered. This method checks if the date needs to be skipped in the for loop.

        input: string (date)
        
        output: True or False.
        """
        if written_date_text in our_parameters.return_skipped_dates():
            return True
        else:
            return False

save_weather_scrape = []

#print(our_parameters.return_skipped_dates())

"""
Testing:
"""
class Test_Stock_Scrape():
    def __init__(self):
        self.tester = Stock_Scraping_Space()
    
    def test_skip_date(self):
        """
        Tests whether we get the right boolean variable for a skipped date and a unskipped date. 
        """
        test_list = []
        expected_value = [True, False]
        skipped_date = '07:40, 24 January 2021'
        not_skipped_date = '19:14, 6 July 2021'
        test_list.append(self.tester.skip_date(skipped_date))
        test_list.append(self.tester.skip_date(not_skipped_date))
        print(test_list == expected_value)
    
    def test_find_next_desired_month(self):
        expected_value = (2021, 'March')
        return expected_value == self.tester.find_next_desired_month((2020, 'August'))
    
    def test_return_edit_list(self):
        """
        A test to see if the correct edit list is returned. Manual inspection has identified this as the correct edit list. 
        """
        td = start_driver()
        scraper.go_to_home(our_parameters.return_exchange_URL(), driver = td)
        return self.tester.return_edit_list(td)
        
    def test_get_month_and_year_from_text(self):
        input = '12:07, 27 August 2021'
        expected_value = ('2021', 'August')
        return expected_value == self.tester.get_month_and_year_from_text('12:07, 27 August 2021')


    def test_scrape_through_edit_list(self):
        td = start_driver()
        scraper.go_to_home(our_parameters.return_exchange_URL(), driver = td)
        t_edit_list = self.test_return_edit_list()
        return self.tester.scrape_through_edit_list(t_edit_list, complete_list=[], list_of_locations=[], completed_months_and_years = []) 
    
    def test_shall_month_be_scraped(self):
        pass
scraper = Stock_Scraping_Space()




test_stock = Test_Stock_Scrape()

#print(test_stock.test_get_month_and_year_from_text())
print(test_stock.test_scrape_through_edit_list()[2])
print(test_stock.test_scrape_through_edit_list()[1])

#test_stock.test_skip_date()
#print(test_stock.test_find_next_desired_month())


from selenium import webdriver
from selenium.webdriver.common.keys import Keys
import time
import pandas as pd


import sys
sys.path.append('..')

def import_packages():
    """
    This is where I can type in all the stuff that I can import with the project.

    TD have this integrated probably so text like above does not apear. 
    """
    from selenium import webdriver
    from selenium.webdriver.common.keys import Keys
    import time
    import pandas as pd

def start_driver():
    """
    Starts a webdriver driver using the chromedriver in the folder. 

    returns driver
    """
    driver = webdriver.Chrome('chromedriver.exe') 
    return driver

def reset_variable(input):
    """
    Depending on the type of input. It will give some default values.
    
    can allow string integer list and boolean.
    returns same type of variable as input. 
    """
    
    if type(input) == str:
        input = ''
    if type(input) == int:
        input = 0
    if type(input) == list:
        input = []
    if type(input) == bool:
        """
        Be weary. This will always default to false (There are more false things than true things initally).
        """
        input = False
    return input


def save_data(name, input_list):
    df = pd.DataFrame(input_list)
    df.to_csv('{name}.csv', index = False, sep= ';')

def create_dict(*args):
    """
    Creates a dictionary with argument names as key names and values as values.

    input valued variables. 
    output dictionary
    """
    return dict({i:eval(i) for i in args})

def seperate_month_and_year(month_and_year):
    """
    month_and_year is normally found as a size 2 tuple. This seperates it into components.

    input length 2 tuple. output length two list.

    TD have this raise a value error if something other than the usual input is inserted. 
    """
    year, month = month_and_year
    dummy_list = [year, month]
    return dummy_list


from selenium import webdriver
from selenium.webdriver.common.keys import Keys
import time
import pandas as pd
from weather_scraper_classes import *
from url_scraper_classes import *
from wikipedia_scraper_classes import *
from main_classes import *
from common_functions import *


wikipedia_xpaths = Wikipedia_Xpath_selection
this_months_dictionary = {'months_rank_list' : [], 'months_exchange_list' : [], 'months_location_list' : [], 'months_market_cap_list' : [], 'months_trade_volume_list' : [], 'current_column' : [], 'table_header_children' : []}
exchange_URL = 'https://en.wikipedia.org/w/index.php?title=List_of_stock_exchanges&action=history'
weather_URL = 'https://www.wunderground.com/history'
certain_stock_exchange_url = 'https://en.wikipedia.org/w/index.php?title=List_of_stock_exchanges&oldid=1040911711'
x = our_parameters.return_URL_sleeping_list()

def scrape_wikipedia():
    """
    The main function that scrapes wikipedia
    """
    wikipedia_scraper = Stock_Scraping_Space()
    return wikipedia_scraper.complete_stock_scrape()

location_urls = []
locations_list = ['New York', 'Tokyo', 'Amsterdam', 'Berlin', 'Madrid']
def scrape_urls():
    """
    The main function that scrapes for the urls
    """
    url_scraper = Location_Url_Scrape(sleeping_amounts = our_parameters.return_URL_sleeping_list())
    driver = webdriver.Chrome('chromedriver.exe')
    return url_scraper.complete_url_scrape(driver = webdriver.Chrome('chromedriver.exe'), list_of_locations = locations_list, location_urls = [])



test_month_location_match = [{'Month': ('2020', 'November'), 'Locations' : ['New York' ,'London', 'Tokyo']},{'Month': ('2020', 'September'), 'Locations' : ['Tokyo' ,'London', 'Hong Kong']}, {'Month': ('2021', 'March'), 'Locations' : ['New York' ,'London', 'Oslo']}]
test_location_key_match = [{'Location Name': 'New York','key': 'KLGA'}, {'Location Name': 'London','key': 'EGLC'}, {'Location Name': 'Tokyo' ,'key': 'RJTT'}, {'Location Name': 'Hong Kong','key': 'VHHH'}, {'Location Name': 'Oslo','key': 'ENGM'}, ]
def scrape_weather():
    """
    The main function that scrapes the weather. 
    """
    weather_scraper = Weather_Scrape(Giant_dataset=[], location_key_match = test_location_key_match, month_location_match = test_month_location_match)
    return weather_scraper.complete_weather_scrape()

#scrape_wikipedia()
#scrape_urls()
scrape_weather()


import sys
from weather_scraper_classes import Weather_Xpath_Selection
sys.path.append('..')
from main_classes import Parameters_selection, Scraping_Space
from weather_scraper_classes import Weather_Xpath_Selection
from common_functions import *
import time 
from selenium import webdriver
from selenium.webdriver.common.keys import Keys

global saved_url_scrape

our_parameters = Parameters_selection
weather_xpaths = Weather_Xpath_Selection


class Location_Url_Scrape(Scraping_Space):
    """
    Methods can be found here to do with the scraping of the URLS.

    This scraping finds the four letter code associated with the location in each URL. There are also methods for outputting this dictionary. 
    """
    def __init__(self, sleeping_amounts):
        self.sleeping_amounts = sleeping_amounts
    
    @staticmethod
    def reset_location_urls(location_urls = []):
        """
        Resets the usuual output list, the list of urls, to the default value.

        output = []
        """
        location_urls = []
        return location_urls

    def location_scrape(self, driver, location, location_urls = []):
        """
        For a single location, it would search the location of the weather homepage, then get the url from the search result. 

        Inputs: The driver we are using, the location we are looking for and the current list of urls (location_urls) which we want outputted.

        Output: Updated list of location urls. 
        """
        driver.get(our_parameters.return_weather_URL())
        time.sleep(self.sleeping_amounts[0])
        search_bar = driver.find_element_by_name(our_parameters.return_history_name())   #This is the search bar
        search_bar.click()

        location = self.location_name_matcher(location)

        search_bar.send_keys(location)  #This types the location in. 
        time.sleep(self.sleeping_amounts[1])
        search_bar.send_keys(Keys.RETURN) #Pressing enter after a delay will send you to a page regarding that location. 
        view_button = driver.find_element_by_xpath(weather_xpaths.get_search_button())
        view_button.click()
        view_button.click()  #This activate button needs to be clicked twice
        time.sleep(self.sleeping_amounts[2])  
        current_url = driver.current_url
        time.sleep(self.sleeping_amounts[3])
        location_urls.append(current_url) 

        time.sleep(self.sleeping_amounts[4])
        driver.get(our_parameters.return_weather_URL())  #Return to searchpage            



        current_url = driver.current_url
        save_data('url_data', location_urls) #The data is being saved here. 
        return location_urls
        


    
    def complete_url_scrape(self, driver, list_of_locations, location_urls):
        """
        For a list of locations, this will scrape through and find a corresponding list of urls. 

        This method mainly is doing the location scraper several times. 

        input: list of locations.
        output: corresponding list of urls
        """
        global saved_url_scrape
        for location in list_of_locations:
            saved_url_scrape = self.location_scrape(driver,  location, location_urls)

        driver.quit()
        saved_url_scrape = location_urls
        return location_urls
    
    @staticmethod
    def location_name_matcher(location):
        """
        This function searches a dictionary to see if the name needs to be changed while searching.

        Most names of locations give their correct destination as their first result. If the location is in this dictionary, it would need to be changed. 

        You need to input the name of a possible location (str)

        As an output, it will changed the name of the location if it is found in the dictionary. If not, it will output the unchanged name.

        """
        diff_name_dict = our_parameters.return_diff_name_dict()
        if location in diff_name_dict:
            location = diff_name_dict[location]
        return location


import sys
sys.path.append('..')
from main_classes import *
from common_functions import *

class Weather_Xpath_Selection():
    """
    This is where the user can change certain xpaths easily without having to find all the usages in the code. 

    These cover the xpaths on the weather website. The function title normally contains the name of the usual variable name making it easy to type in and refer to. 
    These all return strings 

    TD: Make this a subclass of some sort of all encompassing variable selector. 

    """
    def get_search_button():
        return '/html/body/app-root/app-history-search/one-column-layout/wu-header/sidenav/mat-sidenav-container/mat-sidenav-content/div/section/div[2]/div/div/div/div/form/lib-date-selector/div/input'
    def get_high_temp():
        return '/html/body/app-root/app-history/one-column-layout/wu-header/sidenav/mat-sidenav-container/mat-sidenav-content/div/section/div[2]/div[1]/div[3]/div[1]/div/lib-city-history-summary/div/div[2]/table/tbody[1]/tr[1]/td[1]'
    def get_low_temp():
        return '/html/body/app-root/app-history/one-column-layout/wu-header/sidenav/mat-sidenav-container/mat-sidenav-content/div/section/div[2]/div[1]/div[3]/div[1]/div/lib-city-history-summary/div/div[2]/table/tbody[1]/tr[2]/td[1]'
    def get_temp_average():
        return '/html/body/app-root/app-history/one-column-layout/wu-header/sidenav/mat-sidenav-container/mat-sidenav-content/div/section/div[2]/div[1]/div[3]/div[1]/div/lib-city-history-summary/div/div[2]/table/tbody[1]/tr[3]/td[1]'
    def get_precipitation_number():
        return '/html/body/app-root/app-history/one-column-layout/wu-header/sidenav/mat-sidenav-container/mat-sidenav-content/div/section/div[2]/div[1]/div[3]/div[1]/div/lib-city-history-summary/div/div[2]/table/tbody[2]/tr/td[1]'
    def get_dew_point():
        return '/html/body/app-root/app-history/one-column-layout/wu-header/sidenav/mat-sidenav-container/mat-sidenav-content/div/section/div[2]/div[1]/div[3]/div[1]/div/lib-city-history-summary/div/div[2]/table/tbody[3]/tr[1]/td[1]'
    def get_dew_high():
        return '/html/body/app-root/app-history/one-column-layout/wu-header/sidenav/mat-sidenav-container/mat-sidenav-content/div/section/div[2]/div[1]/div[3]/div[1]/div/lib-city-history-summary/div/div[2]/table/tbody[3]/tr[2]/td[1]'
    def get_dew_low():
        return '/html/body/app-root/app-history/one-column-layout/wu-header/sidenav/mat-sidenav-container/mat-sidenav-content/div/section/div[2]/div[1]/div[3]/div[1]/div/lib-city-history-summary/div/div[2]/table/tbody[3]/tr[3]/td[1]'
    def get_dew_average():
        return '/html/body/app-root/app-history/one-column-layout/wu-header/sidenav/mat-sidenav-container/mat-sidenav-content/div/section/div[2]/div[1]/div[3]/div[1]/div/lib-city-history-summary/div/div[2]/table/tbody[3]/tr[4]/td[1]'
    def get_max_wind_speed():
        return '/html/body/app-root/app-history/one-column-layout/wu-header/sidenav/mat-sidenav-container/mat-sidenav-content/div/section/div[2]/div[1]/div[3]/div[1]/div/lib-city-history-summary/div/div[2]/table/tbody[4]/tr[1]/td[1]'
    def get_visibility():
        return '/html/body/app-root/app-history/one-column-layout/wu-header/sidenav/mat-sidenav-container/mat-sidenav-content/div/section/div[2]/div[1]/div[3]/div[1]/div/lib-city-history-summary/div/div[2]/table/tbody[4]/tr[2]/td[1]'
    def get_day_length():
        return '/html/body/app-root/app-history/one-column-layout/wu-header/sidenav/mat-sidenav-container/mat-sidenav-content/div/section/div[2]/div[1]/div[3]/div[1]/div/lib-city-history-summary/div/div[2]/table/tbody[6]/tr[1]/td[1]'

    def get_observations_body():
        """
        This Xpath refers to the table where the timely specific variables occur agaisnt their time of day. 
        """
        return '/html/body/app-root/app-history/one-column-layout/wu-header/sidenav/mat-sidenav-container/mat-sidenav-content/div/section/div[2]/div[1]/div[5]/div[1]/div/lib-city-history-observation/div/div[2]/table/tbody'

#TD This ideally should not be needed here but code wouldn't work without it. Find out how to improve this.  
weather_xpaths = Weather_Xpath_Selection
test_month_location_match = [{'Month': ('2020', 'November'), 'Locations' : ['New York' ,'London', 'Tokyo']},{'Month': ('2020', 'September'), 'Locations' : ['Tokyo' ,'London', 'Hong Kong']}, {'Month': ('2021', 'March'), 'Locations' : ['New York' ,'London', 'Oslo']}]
test_location_key_match = [{'Location Name': 'New York','key': 'KLGA'}, {'Location Name': 'London','key': 'EGLC'}, {'Location Name': 'Tokyo' ,'key': 'RJTT'}, {'Location Name': 'Hong Kong','key': 'VHHH'}, {'Location Name': 'Oslo','key': 'ENGM'}, ]

class Weather_Scrape(Scraping_Space):
    """
    This is the class that does the web scraping for the weather it has many layers.
    """
    def __init__(self, Giant_dataset, location_key_match, month_location_match):
        """
        saved_webscraper is where the data gets stored and saved. 
        """
        super().__init__()
        self.Giant_dataset = Giant_dataset
        self.Giant_dataset = []
        self.location_key_match = location_key_match
        self.saved_weather_scraper = []
        self.month_location_match = month_location_match

    def reset_dataset(self):
        """
        This resets the huge list which is being scraped through. 
        """
        self.Giant_dataset = []
    


    def complete_weather_scrape(self):
        """
        This funciton gets all the data for each month, month by month. It puts all the data into the Giant_datasert
        """
        for months in self.month_location_match:
            self.month_year_weather_scrape(months)


    def month_year_weather_scrape(self, months):
        """
        This scrapes the data for all location for this month 
        """
        current_month_list = []
        current_month_dict = {}
        current_year, current_month = seperate_month_and_year(months['Month'])
        month_info = Month_And_Year(month = current_month, year = current_year)
        month_number = month_info.get_month_number()
        


        current_month_dict['Month'] = (current_month, current_year) #These last two enteries correspond to the month of the year and the year. 
        current_month_dict['Data'] = [] 
        

        for locations in months['Locations']:  #We are finding the key for each location here.
            self.location_and_month_scrape(locations, month_info.get_amount_of_days(), current_year, month_number, current_month_dict, current_month_list)
            save_data(f"{current_month} + {current_year} + info", current_month_dict) # Data is being saved here.
            
            
        self.Giant_dataset.append(current_month_dict)  #Add the month's data to the giant dataset. 



    def location_and_month_scrape(self, locations, amount_of_days, current_year, month_number, current_month_dict, current_month_list):
        """
        This scrapes all the data for this combination of location and month 
        """
        current_key = 'a'

        #Good use for a search function here. 
        for enteries in self.location_key_match:
            if enteries['Location Name'] == locations:
                current_key = enteries['key']    #The key is found.

    
        for i in range(1, amount_of_days + 1):  #This range means for every day of the month
            current_month_list.append(self.day_scrape(i, locations, current_key, current_year, month_number, current_month_dict))
            
        return(current_month_list)
        





    
    def try_daily_observations_scraping(self, current_date_dict, driver):  # This is a fucntion which scrapes from the table in the weather section.
        """
        The attempt to try to scrape the specific daily info
        """
        time_data = []
        try:
            self.daily_observation_scraping_attempt(current_date_dict, driver)

       
        except:
            current_date_dict['Time_Data'] = time_data


    def daily_observation_scraping_attempt(self, current_date_dict, driver):
        """
        The scraping of the specific daily info assuming it is possible. 
        """

        list_of_rows = self.get_all_children(driver, weather_xpaths.get_observations_body())
        time_data = []
        for row in list_of_rows:
            self.get_time_of_day_weather_information(row, time_data)


        current_date_dict['Time_Data'] = time_data
        return(current_date_dict)
    
    def get_time_of_day_weather_information(self, row, time_data):
        """
        This method gets the information for specific times of days throughout the day. 
        """
        time_condition = {} # A dictionary recording data for each time in the day in the table. 
        column_number = 1 # Start with the leftmost column. 

        row_elements = row.find_elements_by_xpath("*")

        for column in row_elements:
            time_data.append(self.time_of_day_column_scrape(column, column_number, time_condition))
            column_number += 1



        time_data.append(time_condition) 
        return time_data      #collect all the time data together and add them to the dictionary. 
        
    @staticmethod
    def time_of_day_column_scrape(column, column_number, time_condition):
        """
        This method goes through a specific row of the table. 
        """
        print(column_number)
        if column_number == 1:
            time_condition['Time'] = column.text # This column is the time column. 
        if column_number == 5: 
            time_condition['Wind_Direction'] = column.text #This column is the wind direction column. 
        if column_number == 10:
            time_condition['Rain_Condition'] = column.text   #This column is the worded rain column. 
        return(time_condition)
        

    def day_scrape(self, i, locations, current_key, current_year, month_number, current_month_dict):
        """
        This scrapes all the information for a single day and location. 
        """
        time.sleep(our_parameters.return_weather_sleeping_list()[0]) 

        driver = webdriver.Chrome('chromedriver.exe')
        current_date_dict = {}
        current_date_dict['day_and_location'] = {'day': i,'location': locations}  #Input the independent variables into the dictionary

        desired_url =  f"https://www.wunderground.com/history/daily/{current_key}/date/{current_year}-{month_number}-{i}"
        driver.get(desired_url)

        time.sleep(our_parameters.return_weather_sleeping_list()[1])

        self.trying_scraping(current_date_dict, driver)   #We are using the two previous functions for scraping these pages
        self.try_daily_observations_scraping(current_date_dict, driver)
        print(current_date_dict)
        driver.close
        current_month_dict['Data'].append(current_date_dict)   #Add all this date to the data value of this month   

    @staticmethod
    def trying_scraping(current_date_dict, driver):  # This function tries scraping from the locations on the weather website. These are try statements so if a page has no info, no info is added.
        """
        For different Xpaths, this method goes one by one in trying to get the data from them. 
        """
        try:
       
            high_temp = driver.find_element_by_xpath('/html/body/app-root/app-history/one-column-layout/wu-header/sidenav/mat-sidenav-container/mat-sidenav-content/div/section/div[2]/div[1]/div[3]/div[1]/div/lib-city-history-summary/div/div[2]/table/tbody[1]/tr[1]/td[1]')
            current_date_dict['High_Temp'] = high_temp.text
        except:
            pass
        
        try:
            low_temp = driver.find_element_by_xpath('/html/body/app-root/app-history/one-column-layout/wu-header/sidenav/mat-sidenav-container/mat-sidenav-content/div/section/div[2]/div[1]/div[3]/div[1]/div/lib-city-history-summary/div/div[2]/table/tbody[1]/tr[2]/td[1]')
            current_date_dict['Low_Temp'] = low_temp.text
        except:
            pass

        try:
            day_average = driver.find_element_by_xpath('/html/body/app-root/app-history/one-column-layout/wu-header/sidenav/mat-sidenav-container/mat-sidenav-content/div/section/div[2]/div[1]/div[3]/div[1]/div/lib-city-history-summary/div/div[2]/table/tbody[1]/tr[3]/td[1]')
            current_date_dict['Day_Average_Temp'] = day_average.text
        except:
            pass

        try:
            precipitation_number = driver.find_element_by_xpath('/html/body/app-root/app-history/one-column-layout/wu-header/sidenav/mat-sidenav-container/mat-sidenav-content/div/section/div[2]/div[1]/div[3]/div[1]/div/lib-city-history-summary/div/div[2]/table/tbody[2]/tr/td[1]')
            current_date_dict['Precipitation_Number'] = precipitation_number.text
        except:
            pass

        try:
            dew_point = driver.find_element_by_xpath('/html/body/app-root/app-history/one-column-layout/wu-header/sidenav/mat-sidenav-container/mat-sidenav-content/div/section/div[2]/div[1]/div[3]/div[1]/div/lib-city-history-summary/div/div[2]/table/tbody[3]/tr[1]/td[1]')
            current_date_dict['Dew_point'] = dew_point.text
        except:
            pass

        try:
            dew_high = driver.find_element_by_xpath('/html/body/app-root/app-history/one-column-layout/wu-header/sidenav/mat-sidenav-container/mat-sidenav-content/div/section/div[2]/div[1]/div[3]/div[1]/div/lib-city-history-summary/div/div[2]/table/tbody[3]/tr[2]/td[1]')
            current_date_dict['Dew_High'] = dew_high.text
        except:
            pass

        try:
            dew_low = driver.find_element_by_xpath('/html/body/app-root/app-history/one-column-layout/wu-header/sidenav/mat-sidenav-container/mat-sidenav-content/div/section/div[2]/div[1]/div[3]/div[1]/div/lib-city-history-summary/div/div[2]/table/tbody[3]/tr[3]/td[1]')
            current_date_dict['Dew_Low'] = dew_low.text
        except:
            pass
        
        try:
            dew_average = driver.find_element_by_xpath('')
            current_date_dict['Dew_Average'] = dew_average.text
        except:
            pass
        
        try:
            max_wind_speed = driver.find_element_by_xpath('/html/body/app-root/app-history/one-column-layout/wu-header/sidenav/mat-sidenav-container/mat-sidenav-content/div/section/div[2]/div[1]/div[3]/div[1]/div/lib-city-history-summary/div/div[2]/table/tbody[4]/tr[1]/td[1]')
            current_date_dict['Max_Wind_Speed'] = max_wind_speed.text
        except:
            pass

        try:
            visibility = driver.find_element_by_xpath('/html/body/app-root/app-history/one-column-layout/wu-header/sidenav/mat-sidenav-container/mat-sidenav-content/div/section/div[2]/div[1]/div[3]/div[1]/div/lib-city-history-summary/div/div[2]/table/tbody[4]/tr[2]/td[1]')
            current_date_dict['Visibility'] = visibility.text
        except:
            pass

        try:
            day_length = driver.find_element_by_xpath('/html/body/app-root/app-history/one-column-layout/wu-header/sidenav/mat-sidenav-container/mat-sidenav-content/div/section/div[2]/div[1]/div[3]/div[1]/div/lib-city-history-summary/div/div[2]/table/tbody[6]/tr[1]/td[1]')
            current_date_dict['Day_length'] = day_length.text
        except:
            pass



import sys
sys.path.append('..')
from selenium import webdriver
from selenium.webdriver.common.keys import Keys
import time
import pandas as pd
from test_common_functions import *




class Non_duplicate_lists():
    """
    This class is for lists that we wouldn't to have duplicates unless very specifically allowed. 

    TD (generally). thoroughly make sure weather some of these lists would be better off as dictionaries as they 
    """
    def __init__(self, input_list = []):
        self.input_list = input_list


    def reset_list(self):
        """
        Empties the list

        output: list
        """
        self.input_list = []
    
    @staticmethod
    def add_if_no_duplication(input_list, new_entry):
        """
        This function adds a new entry to the list only if it is not existing inside already. 


        input: List, List_entry (any acceptable list variable)
        output: List
        """
        item_found = False
        for items in input_list:
            if items == new_entry:
                item_found = True
        if item_found == False:
            input_list.append(new_entry)     
        return input_list   

    def alphabetise(self):
        """
        Orders the list. this means alphabetised list for strings.  

        input, output: list of strings  (optional)

        TD: incorpate this into searching algorithms. 
        """
        self.input_list.sort
    
    def return_list(self):
        return self.input_list


"""
Test functions
"""
def test_add_if_no_duplicant():
    test_list = []
    expected_value = [1, 2]
    non_duplicate_list = Non_duplicate_lists()
    non_duplicate_list.add_if_no_duplication(test_list, new_entry = 1)
    non_duplicate_list.add_if_no_duplication(test_list, new_entry = 1)
    non_duplicate_list.add_if_no_duplication(test_list, new_entry = 2)
    print(expected_value == test_list)
    
#test_add_if_no_duplicant()

#TD make it so that the code works without the above variable(s) and function(s)






class Rigid_Parameters_Selection():
    """
    These variables should never be changed. 

    For example there will very likely never be a different list of months.

    TD Implement this properly so it is private
    
    """
    def return_list_of_month_names():
        """
        return a list of all the months

        output: list
        """
        return ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December']
    def return_month_number_match():
        """
        returns a dictionary matching the month with the position it has in a year.

        outputs: Dictionary with string key and integer output. 
        """
        return [{'Month': 'January', 'Number': '1'},
                      {'Month': 'February', 'Number': '2'}, 
                      {'Month': 'March', 'Number': '3'}, 
                      {'Month': 'April', 'Number': '4'}, 
                      {'Month': 'May', 'Number': '5'}, 
                      {'Month': 'June', 'Number': '6'},
                      {'Month': 'July', 'Number': '7'},
                      {'Month': 'August', 'Number': '8'},
                      {'Month': 'September', 'Number': '9'},
                      {'Month': 'October', 'Number': '10'},
                      {'Month': 'November', 'Number': '11'},
                      {'Month': 'December', 'Number': '12'},] 
rigid_parameters = Rigid_Parameters_Selection
#TD This ideally should not be needed here but code wouldn't work without it. Find out how to improve this.         


"""
Testing
"""

def test_rigid_parameters():
    """
    As this is a simple class. This should be enough. 
    """
    expected_value = ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December']
    rigid_tester = Rigid_Parameters_Selection
    output = rigid_tester.return_list_of_month_names() #Expected output is the list of the month names e.g ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December']
    print(expected_value == output)

#test_rigid_parameters




class Parameters_selection():
    """
    These parameters are likely to be changed. They can all be found here without digging through the code.

    The function names usually contatin the name of the variable being used.  

    outputs vary. 
    """
    def return_amount_of_ranks():
        return 16
    def return_date_text_identifier():
        return 'mw-changeslist-date'
    def return_end_date():
        return ['16:08, 6 August 2020']
    def return_exchange_URL():
        return 'https://en.wikipedia.org/w/index.php?title=List_of_stock_exchanges&action=history'
    def return_format_change_dates():
        return ['02:17, 22 April 2021']
    def return_history_name():
        return 'historySearch'
    def return_skipped_dates():
        return ['07:40, 24 January 2021']
    def return_weather_URL():
        return 'https://www.wunderground.com/history' 
    def return_URL_sleeping_list():
        return [4, 4, 1, 2, 2]
    def return_weather_sleeping_list():
        return [4, 4]
    def return_diff_name_dict():
        return {'Hong Kong' : 'Kowloon', 'Mexico City': 'IMEXICOC52'}
    def return_correct_link_length():
        return 36

    def return_desired_scraped_months():
        return [('2021', 'September'),('2021', 'August'),('2021', 'July'),
            ('2021', 'June'),('2021', 'May'),('2021', 'April'),
            ('2021', 'March'),('2021', 'February'),('2021', 'January'),
            ('2020', 'December'), ('2020', 'November'),('2020', 'October'),
            ('2020', 'September'),('2020', 'August') ]

#TD This ideally should not be needed here but code wouldn't work without it. Find out how to improve this.  
"""
Testing for class
"""
def test_return_desired_scraped_months():
    test_parameters = Parameters_selection()
    output = test_parameters.return_desired_scraped_months
    expected_value = [(2021, 'September'),(2021, 'August'),(2021, 'July'),
            (2021, 'June'),(2021, 'May'),(2021, 'April'),
            (2021, 'March'),(2021, 'February'),(2021, 'January'),
            (2020, 'December'), (2020, 'November'),(2020, 'October'),
            (2020, 'September'),(2020, 'August') ]
    print(expected_value == output)

#test_return_desired_scraped_months
#print(our_parameters.return_exchange_URL())   


class Saved_values():
    
    def __init__():
        pass
    
    @staticmethod
    def initalise_values():
        global saved_url_scrape, saved_wikipedia_scrape, saved_weather_scrape
        saved_url_scrape = []
        saved_wikipedia_scrape = []
        saved_weather_scrape = []
    
    @staticmethod
    def reset_variable(input):
        input 









class Matching_Dictionary():
    """
    Little used class for future incorportion.  This is a list of dictionaries where 'the left values' the keys are strongly linked to 'the right values' the values. 

    TD incorporate this into code more. A dictionary of dicitonaries woudld be way faster.  

    TD Comprehensive testing regarding other inputted lists.
    """
    def __init__(self, left_key, right_key, input_list):
        """
        The different variables used here .

        The input_list would be structured like the following:[{'Month': 'January', 'Number': '1'},
                      {'Month': 'February', 'Number': '2'}, 
                      {'Month': 'March', 'Number': '3'}, 
                      {'Month': 'April', 'Number': '4'}, 
                      {'Month': 'May', 'Number': '5'}, 
                      {'Month': 'June', 'Number': '6'},
                      {'Month': 'July', 'Number': '7'},
                      {'Month': 'August', 'Number': '8'},
                      {'Month': 'September', 'Number': '9'},
                      {'Month': 'October', 'Number': '10'},
                      {'Month': 'November', 'Number': '11'},
                      {'Month': 'December', 'Number': '12'},] 
        I.E a list containg dictionaries with two keys each. 


        TD it looks like the left key and the right key can possibly be derived somehow. 
        """
        self.left_key = left_key
        self.right_key = right_key
        self.input_list = input_list

    def change_list(self, input_list):
        """
        If you want the input list to change 
        """
        self.input_list = input_list


    def add_key():
        """
        TD
        """
        pass

    @classmethod
    def __iter__(self):
        return self

    def get_rightvalue_from_leftvalue(self, left_value):
        """
        This function gets us the matching value from the left value for any key which appears in any of the dictonaries. 

        In the only current implentation of this function, the code searches the dictonaries for one which contain the month we are looking for. 
        It then returns its correpsonging value. 

        TD add failure result. 

        """
        
        for dictionaries in self.input_list:
            if dictionaries[self.left_key] == left_value:
                return dictionaries[self.right_key]
        

    @classmethod
    def alphatbetise_by_left_key(cls):
        pass

    def return_inputs(self):
        """
        Returns the inputs. Incase place of inputs is hard to find. 
        """
        return [self.left_key, self.right_key, self.input_list]

"""
Testing
"""
def test_matching_dictionary():
    matching_test = Matching_Dictionary(left_key = 'Month', right_key = 'Number', input_list = rigid_parameters.return_month_number_match())   #The month number matcher is the dicitonary that appears in the class description.
    test_list = []
    test_list.append(matching_test.return_inputs())  #This should return the inputs we just put in. 
    test_list.append(matching_test.get_rightvalue_from_leftvalue(left_value = 'May')) #Expected results for these four: 5, 3, 6, 7
    test_list.append(matching_test.get_rightvalue_from_leftvalue(left_value = 'March'))
    test_list.append(matching_test.get_rightvalue_from_leftvalue(left_value = 'June'))
    test_list.append(matching_test.get_rightvalue_from_leftvalue(left_value = 'July'))
    expected_value = [['Month', 'Number', [{'Month': 'January', 'Number': '1'}, {'Month': 'February', 'Number': '2'}, 
        {'Month': 'March', 'Number': '3'}, {'Month': 'April', 'Number': '4'}, 
        {'Month': 'May', 'Number': '5'}, {'Month': 'June', 'Number': '6'}, 
        {'Month': 'July', 'Number': '7'}, {'Month': 'August', 'Number': '8'}, 
        {'Month': 'September', 'Number': '9'}, {'Month': 'October', 'Number': '10'}, 
        {'Month': 'November', 'Number': '11'}, {'Month': 'December', 'Number': '12'}]], '5', '3', '6', '7']
    print(test_list == expected_value)
#test_matching_dictionary()

class Month_And_Year():
    """
    A combination of month and year.

    The month is as a word (i.e 'May'). The year is as a four digit string (i.e 2010)
    """
    def __init__(self, month, year):
        """
        month and year are defined here
        """
        self.month = month
        self.year = year
    def get_month_number(self):
        """
        This functions gets the position of a month in a year based on its name

        input: string
        output: integer (1 to 12)
        """
        month_number_matcher = Matching_Dictionary(left_key = 'Month', right_key = 'Number', input_list = rigid_parameters.return_month_number_match())
        return month_number_matcher.get_rightvalue_from_leftvalue( left_value = self.month)

    def get_amount_of_days(self):
        """
        This function finds the amount of days in a month given the name of the month.

        output: integer (28 to 31)
        
        """
        if self.month in ('July', 'May', 'March', 'January', 'December', 'October','August'):  
            amount_of_days = 31
        elif self.month in ('June', 'April', 'November', 'September'): 
            amount_of_days = 30
        elif int(self.year) %4 == 0: #A test for whether the year is a leap year or not. This is sufficient logic in the span of dates we are looking at. 
            amount_of_days = 29
        else:
            amount_of_days = 28
        return amount_of_days
    def return_month(self):
        """
        Seems superflous but can maybe come in handy
        """
        return self.month


    def return_year(self):
        """
        Seems superflous but can maybe come in handy. Good for testing initialisation 
        """
        return self.year


    def return_month_and_year_tuple(self):
        """
        This method joins the month and year together.
        """
        return (self.year, self.month)

"""
Testing
"""

def test_month_and_year():
    march_2020 = Month_And_Year(month = 'March', year = 2020)
    feb_2021 = Month_And_Year(month = 'February', year = 2021)
    feb_2020 = Month_And_Year(month = 'February', year = 2020)

    
    test_month_number_return_list = []
    test_amount_of_days_rl = []
    test_tuple_rl = []

    expected_number_return_list = ['3', '2']
    expected_tuple = ['March', 2020, (2020, 'March')]
    expected_amount_of_days = ['3', '2', 28, 29, 31]
    expected_month_numbers = ['3', '2']

    test_amount_of_days_rl.append(march_2020.get_month_number()) #Expected values: 3 and 2.
    test_amount_of_days_rl.append(feb_2020.get_month_number())
    print(test_amount_of_days_rl == expected_number_return_list)
    
    test_tuple_rl.append(march_2020.return_month())
    test_tuple_rl.append(march_2020.return_year()) #Expected values for these three are March, 2020 and (2020, March) respectively
    test_tuple_rl.append(march_2020.return_month_and_year_tuple())
    print(test_tuple_rl == expected_tuple)

    test_amount_of_days_rl.append(feb_2021.get_amount_of_days())
    test_amount_of_days_rl.append(feb_2020.get_amount_of_days())
    test_amount_of_days_rl.append(march_2020.get_amount_of_days())   #This is where a test for these days of the month. These all give the correct amount of days
    print(test_amount_of_days_rl == expected_amount_of_days)

    test_month_number_return_list.append(march_2020.get_month_number())
    test_month_number_return_list.append(feb_2021.get_month_number())
    print(test_month_number_return_list == expected_month_numbers)

#test_month_and_year()




class Scraping_Space():
    """
    General scraping mechanism. 
    
    This includes functions common to different scrapers. 
    """
    def __init__(self):
        pass


    @staticmethod
    def get_all_children(current_driver, xpath):
        """
        This is for tables and such. It creates a list of the webelements from a table (or over encompassing web object).

        This usability of this function relies heavily. On how the output is used. 

        TD. add categories to the different web elements being outputted so the outputs are less likely to break later. 

        input: xpath
        output: list of web elements.
        """
        dummy_var = current_driver.find_element_by_xpath(xpath)
        dummy_list = dummy_var.find_elements_by_xpath("*")
        return dummy_list


    @staticmethod
    def outputting_dict(list_of_keys, list_of_data):
        """
        Outputs a dictionary created from two lists resembelming the keys and values specifically. 

        TD. Incorporate this into code. 
        """
        output_dict = {}
        looper = 0 
        for names in list_of_keys:
            output_dict[names] = list_of_data[looper]
            looper += 1
        return output_dict
        
    @staticmethod
    def go_to_link(current_driver, link):
        """
        Make the driver go to the inputted link. 
        """
        current_driver.get(link)
    
    @staticmethod
    def close_driver(driver):
        """
        closes the driver.

        TD: Check this is unused then delete, redundant.
        """
        driver.close
"""
Testing
"""


#test_scraping_space = Test_Scraping_Space()
#test_driver = test_start_driver()  #These first two lines test the functionality in opening the driver. 
#test_scraping_space.go_to_link(test_driver, 'https://en.wikipedia.org/w/index.php?title=List_of_stock_exchanges&oldid=1040911711') #These two tests succesfully open the driver and go to the wikipedia link. 

# The following test the get all children method. This should return a list of web elements. The expected result is a large amount of '<class 'selenium.webdriver.remote.webelement.WebElement'> being printed
#test_list = test_scraping_space.get_all_children(current_driver = test_driver, xpath = '//*[@id="mw-content-text"]/div[1]/table[2]/tbody/tr[1]')
#for webelement in test_list:
 #   print(type(webelement))
#for link in test_list:
#Found_date = link.find_element_by_class_name('mw-changeslist-date')
# Found_date_text = Found_date.text
# # print(Found_date.text)

#test_list_of_keys = ['First', 'Second', 'Third', 'Fourth', 'Fifth', 'Sixth']
#test_list_of_data = [0, [1, 2], 'hello', 'hello, again', [12, 23, 34], 5]

#An independent test for the this method. Expected output = {'First': 0, 'Second': [1, 2], 'Third': 'hello', 'Fourth': 'hello, again', 'Fifth': [12, 23, 34], 'Sixth': 5}
#print(test_scraping_space.outputting_dict(test_list_of_keys, test_list_of_data))

#This section tests the close driver function. If it works then the go to link method won't do anything
#test_scraping_space.close_driver(test_driver)
#test_scraping_space.go_to_link(test_driver, 'https://en.wikipedia.org/w/index.php?title=List_of_stock_exchanges&oldid=1040911711'



from setuptools import setup

setup(
    name='url_scraper',
    version='1.0.1',    
    description='scrapes the weather page for urls',
    url='https://github.com/hjasmith1996/Project_2-v2.git',
    author='Harry Smith',
    author_email='hjasmith1996@gmail.com',
    license='BSD 2-clause',
    packages=['Project'],
    install_requires=['mpi4py>=2.0',
                      'numpy', 'selenium', 'pandas', 'beautifulsoup4', 'requests'                    
                      ],

    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',  
        'Operating System :: POSIX :: Linux',        
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)


from main_classes import Matching_Dictionary
    
class Column_Condition_Matcher(Matching_Dictionary):
    """
    Unused class which would have given values in a table a place in a dictonary with the column header as key name. 
    """
    def __init__(self, input_list):
        self.left_key = 'Coloumn number'
        self.right_key = 'Property'
        self.input_list = input_list
    def return_new_dict(self):
        output_dict = {}
        for dicts in self.input_list:
            pass
        return output_dict

class Leaf_Xpath_Match(Matching_Dictionary):
    """ 
    another unused class in which the children with no descendants in a webpage match with their xpath. 

    The hope for this was to make a DPS algorithm to search through all xpaths in a webpage. 
    """
    def __init__(self, input_list):
        self.left_key = 'Property'
        self.right_key = 'Xpath'
        self.input_list = input_list
    def create_new_dict_from_scrape(self, driver):
        scraped_data = {}
        for item in self.input_list:
            print(item[self.left_key])
            print(item[self.right_key])
            a =  driver.find_element_by_xpath({item[self.right_key]})
            scraped_data[{item[self.left_key]}] = a.text
            print(item[self.left_key])
            print(item[self.right_key])
        return scraped_data


import sys
sys.path.append('..')
from selenium import webdriver
from selenium.webdriver.common.keys import Keys
import time
import pandas as pd
from common_functions import *


month_number_match = [{'Month': 'January', 'Number': '1'},
                      {'Month': 'February', 'Number': '2'}, 
                      {'Month': 'March', 'Number': '3'}, 
                      {'Month': 'April', 'Number': '4'}, 
                      {'Month': 'May', 'Number': '5'}, 
                      {'Month': 'June', 'Number': '6'},
                      {'Month': 'July', 'Number': '7'},
                      {'Month': 'August', 'Number': '8'},
                      {'Month': 'September', 'Number': '9'},
                      {'Month': 'October', 'Number': '10'},
                      {'Month': 'November', 'Number': '11'},
                      {'Month': 'December', 'Number': '12'},]

#TD make it so that the code works without the above variable(s) and function(s)



#TD make sure there are no unused imports here. 

class Saved_values():
    """
    These are some global variables which we make global variables so we don't lose them easily. 
    """
    def __init__():
        pass
    
    @staticmethod
    def initalise_values():
        global saved_url_scrape, saved_wikipedia_scrape, saved_weather_scrape
        saved_url_scrape = []
        saved_wikipedia_scrape = []
        saved_weather_scrape = []
    
    @staticmethod
    def get_test_variables():
        global saved_url_scrape, saved_wikipedia_scrape, saved_weather_scrape
        saved_url_scrape = ['https://www.wunderground.com/history/daily/KLGA/date/2021-9-9', 'https://www.wunderground.com/history/daily/EGLC/date/2021-9-9', 'https://www.wunderground.com/history/daily/MMMX/date/2021-9-9', 'https://www.wunderground.com/history/daily/VHHH/date/2021-9-9']
        saved_wikipedia_scrape = []
        saved_weather_scrape = []    
    @staticmethod
    def reset_variable(input):
        input = []
        return input



class Non_duplicate_lists:
    """
    This class is for lists that we wouldn't to have duplicates unless very specifically allowed. 

    TD (generally). thoroughly make sure weather some of these lists would be better off as dictionaries as they 
    """
    def __init__(self, input_list = []):
        self.input_list = input_list


    def reset_list(self):
        """
        Empties the list

        output: list
        """
        self.input_list = []
    
    @staticmethod
    def add_if_no_duplication(input_list, new_entry):
        """
        This function adds a new entry to the list only if it is not existing inside already. 


        input: List, List_entry (any acceptable list variable)
        output: List
        """
        item_found = False
        for items in input_list:
            if items == new_entry:
                item_found = True
        if item_found == False:
            input_list.append(new_entry)     
        return input_list   

    def alphabetise(self):
        """
        Orders the list. this means alphabetised list for strings.  

        input, output: list of strings  (optional)

        TD: incorpate this into searching algorithms. 
        """
        self.input_list.sort
    
    def return_list(self):
        return self.input_list
            








class Rigid_Parameters_Selection():
    """
    These variables should never be changed. 

    For example there will very likely never be a different list of months.

    TD Implement this properly so it is private
    
    """
    def return_list_of_month_names():
        """
        return a list of all the months

        output: list
        """
        return ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December']
    def return_month_number_match():
        """
        returns a dictionary matching the month with the position it has in a year.

        outputs: Dictionary with string key and integer output. 
        """
        return [{'Month': 'January', 'Number': '1'},
                      {'Month': 'February', 'Number': '2'}, 
                      {'Month': 'March', 'Number': '3'}, 
                      {'Month': 'April', 'Number': '4'}, 
                      {'Month': 'May', 'Number': '5'}, 
                      {'Month': 'June', 'Number': '6'},
                      {'Month': 'July', 'Number': '7'},
                      {'Month': 'August', 'Number': '8'},
                      {'Month': 'September', 'Number': '9'},
                      {'Month': 'October', 'Number': '10'},
                      {'Month': 'November', 'Number': '11'},
                      {'Month': 'December', 'Number': '12'},] 

#TD This ideally should not be needed here but code wouldn't work without it. Find out how to improve this.         
rigid_parameters = Rigid_Parameters_Selection

class Parameters_selection():
    """
    These parameters are likely to be changed. They can all be found here without digging through the code.

    The function names usually contatin the name of the variable being used.  

    outputs vary. 
    """
    def return_amount_of_ranks():
        return 16
    def return_date_text_identifier():
        return 'mw-changeslist-date'
    def return_end_date():
        return ['16:08, 6 August 2020']
    def return_exchange_URL():
        return 'https://en.wikipedia.org/w/index.php?title=List_of_stock_exchanges&action=history'
    def return_format_change_dates():
        return ['02:17, 22 April 2021']
    def return_history_name():
        return 'historySearch'
    def return_skipped_dates():
        return ['07:40, 24 January 2021']
    def return_weather_URL():
        return 'https://www.wunderground.com/history' 
    def return_URL_sleeping_list():
        return [4, 4, 1, 2, 2]
    def return_weather_sleeping_list():
        return [4, 4]
    def return_diff_name_dict():
        return {'Hong Kong' : 'Kowloon', 'Mexico City': 'IMEXICOC52'}
    def return_correct_link_length():
        return 36

    def return_desired_scraped_months():
        return [('2021', 'September'),('2021', 'August'),('2021', 'July'),
            ('2021', 'June'),('2021', 'May'),('2021', 'April'),
            ('2021', 'March'),('2021', 'February'),('2021', 'January'),
            ('2020', 'December'), ('2020', 'November'),('2020', 'October'),
            ('2020', 'September'),('2020', 'August') ]

#TD This ideally should not be needed here but code wouldn't work without it. Find out how to improve this.  
our_parameters = Parameters_selection    
                









#TD This ideally should not be needed here but code wouldn't work without it. Find out how to improve this.              








class Matching_Dictionary():
    """
    Little used class for future incorportion.  This is a list of dictionaries where 'the left values' the keys are strongly linked to 'the right values' the values. 

    TD incorporate this into code more. A dictionary of dicitonaries woudld be way faster.  

    TD Comprehensive testing regarding other inputted lists.
    """
    def __init__(self, left_key, right_key, input_list):
        """
        The different variables used here .

        The input_list would be structured like the following:[{'Month': 'January', 'Number': '1'},
                      {'Month': 'February', 'Number': '2'}, 
                      {'Month': 'March', 'Number': '3'}, 
                      {'Month': 'April', 'Number': '4'}, 
                      {'Month': 'May', 'Number': '5'}, 
                      {'Month': 'June', 'Number': '6'},
                      {'Month': 'July', 'Number': '7'},
                      {'Month': 'August', 'Number': '8'},
                      {'Month': 'September', 'Number': '9'},
                      {'Month': 'October', 'Number': '10'},
                      {'Month': 'November', 'Number': '11'},
                      {'Month': 'December', 'Number': '12'},] 
        I.E a list containg dictionaries with two keys each. 


        TD it looks like the left key and the right key can possibly be derived somehow. 
        """
        self.left_key = left_key
        self.right_key = right_key
        self.input_list = input_list

    def change_list(self, input_list):
        """
        If you want the input list to change 
        """
        self.input_list = input_list


    def add_key():
        """
        TD
        """
        pass

    @classmethod
    def __iter__(self):
        return self

    def get_rightvalue_from_leftvalue(self, left_value):
        """
        This function gets us the matching value from the left value for any key which appears in any of the dictonaries. 

        In the only current implentation of this function, the code searches the dictonaries for one which contain the month we are looking for. 
        It then returns its correpsonging value. 

        TD add failure result. 

        """
        
        for dictionaries in self.input_list:
            if dictionaries[self.left_key] == left_value:
                return dictionaries[self.right_key]
        

    @classmethod
    def alphatbetise_by_left_key(cls):
        pass

    def return_inputs(self):
        """
        Returns the inputs. Incase place of inputs is hard to find. 
        """
        return [self.left_key, self.right_key, self.input_list]




class Month_And_Year():
    """
    A combination of month and year.

    The month is as a word (i.e 'May'). The year is as a four digit string (i.e 2010)
    """
    def __init__(self, month, year):
        """
        month and year are defined here
        """
        self.month = month
        self.year = year
    def get_month_number(self):
        """
        This functions gets the position of a month in a year based on its name

        input: string
        output: integer (1 to 12)
        """
        month_number_matcher = Matching_Dictionary(left_key = 'Month', right_key = 'Number', input_list = rigid_parameters.return_month_number_match())
        return month_number_matcher.get_rightvalue_from_leftvalue( left_value = self.month)

    def get_amount_of_days(self):
        """
        This function finds the amount of days in a month given the name of the month.

        output: integer (28 to 31)
        
        """
        if self.month in ('July', 'May', 'March', 'January', 'December', 'October','August'):  
            amount_of_days = 31
        elif self.month in ('June', 'April', 'November', 'September'): 
            amount_of_days = 30
        elif int(self.year) %4 == 0: #A test for whether the year is a leap year or not. This is sufficient logic in the span of dates we are looking at. 
            amount_of_days = 29
        else:
            amount_of_days = 28
        return amount_of_days
    def return_month(self):
        """
        Seems superflous but can maybe come in handy
        """
        return self.month


    def return_year(self):
        """
        Seems superflous but can maybe come in handy. Good for testing initialisation 
        """
        return self.year


    def return_month_and_year_tuple(self):
        """
        This method joins the month and year together.
        """
        return (self.year, self.month)




class Scraping_Space():
    """
    General scraping mechanism. 
    
    This includes functions common to different scrapers. 
    """
    def __init__(self, sleeping_amounts):
        self.sleeping_amounts = sleeping_amounts
    @staticmethod


    def get_all_children(driver, xpath):
        """
        This is for tables and such. It creates a list of the xpaths of a table (or over encompassing web object).

        input: xpath
        output: list of xpaths 
        """
        dummy_var = driver.find_element_by_xpath(xpath)
        dummy_list = dummy_var.find_elements_by_xpath("*")
        return dummy_list


    @staticmethod
    def outputting_dict(list_of_keys, list_of_data):
        """
        Outputs a dictionary created from two lists resembelming the keys and values specifically. 

        TD. Incorporate this into code. 
        """
        output_dict = {}
        for i in list_of_keys:
            output_dict[{list_of_keys[i]}] = list_of_data[i]
        
    def go_to_link(self, driver, link):
        """
        Make the driver go to the inputted link. 
        """
        driver.get(link)

    def close_driver(driver):
        """
        closes the driver.

        TD: Check this is unused then delete, redundant.
        """
        driver.close

class Scraping_Space():
    """
    General scraping mechanism. 
    
    This includes functions common to different scrapers. 
    """
    def __init__(self):
        pass


    @staticmethod
    def get_all_children(current_driver, xpath):
        """
        This is for tables and such. It creates a list of the webelements from a table (or over encompassing web object).

        This usability of this function relies heavily. On how the output is used. 

        TD. add categories to the different web elements being outputted so the outputs are less likely to break later. 

        input: xpath
        output: list of web elements.
        """
        dummy_var = current_driver.find_element_by_xpath(xpath)
        dummy_list = dummy_var.find_elements_by_xpath("*")
        return dummy_list


    @staticmethod
    def outputting_dict(list_of_keys, list_of_data):
        """
        Outputs a dictionary created from two lists resembelming the keys and values specifically. 

        TD. Incorporate this into code. 
        """
        output_dict = {}
        looper = 0 
        for names in list_of_keys:
            output_dict[names] = list_of_data[looper]
            looper += 1
        return output_dict
        
    @staticmethod
    def go_to_link(current_driver,link):
        """
        Make the driver go to the inputted link. 
        """
        current_driver.get(link)
    
    @staticmethod
    def close_driver(driver):
        """
        closes the driver.

        TD: Check this is unused then delete, redundant.
        """
        driver.close        
            


























































































































class Matching_Dictionary():
    """
    Unused class for future incorportion.  Dictionaries where key and value are very strongly linked and used to identify eachother. 

    Would have been used several times in project. 

    TD incorporate this into code. 
    """
    def __init__(self, left_key, right_key, input_list):
        self.left_key = left_key
        self.right_key = right_key
        self.input_list = input_list
    def add_list(self, input_list):
        self.input_list = input_list
    def add_key():
        pass
    @classmethod
    def __iter__(self):
        return self

    def get_rightvalue_from_leftvalue(self,  left_value):
        for dictionaries in self.input_list:
            if dictionaries[self.left_key] == left_value:
                return dictionaries[self.right_key]

    @classmethod
    def alphatbetise_by_left_key(cls):
        pass


class Column_Condition_Matcher(Matching_Dictionary):
    def __init__(self):
        super().__init__()
    """
    Unused class which would have given values in a table a place in a dictonary with the column header as key name. 
    """
    def __init__(self, input_list):
        self.left_key = 'Coloumn number'
        self.right_key = 'Property'
        self.input_list = input_list
    def return_new_dict(self):
        output_dict = {}
        for dicts in self.input_list:
            pass
        return output_dict
    

class Saved_Data():
    pass

class Unusual_Locations_For_Searching():
    pass

class Leaf_Xpath_Match(Matching_Dictionary):
    """ 
    another unused class in which the children with no descendants in a webpage match with their xpath. 

    The hope for this was to make a DPS algorithm to search through all xpaths in a webpage. 
    """
    def __init__(self):
        super().__init__()
    def __init__(self, input_list):
        self.left_key = 'Property'
        self.right_key = 'Xpath'
        self.input_list = input_list
    def create_new_dict_from_scrape(self, driver):
        scraped_data = {}
        for item in self.input_list:
            print(item[self.left_key])
            print(item[self.right_key])
            a =  driver.find_element_by_xpath({item[self.right_key]})
            scraped_data[{item[self.left_key]}] = a.text
            print(item[self.left_key])
            print(item[self.right_key])
        return scraped_data




#month_number_matcher = Matching_Dictionary(left_key = 'Month', right_key = 'Number', input_list = month_number_match)

#print(month_number_matcher.get_rightvalue_from_leftvalue( left_value = 'September'))
class Month_And_Year():
    def __init__(self, month, year):
        self.month = month
        self.year = year
    def get_month_number(self):
        month_number_matcher = Matching_Dictionary(left_key = 'Month', right_key = 'Number', input_list = month_number_match)
        return month_number_matcher.get_rightvalue_from_leftvalue( left_value = self.month)
    def get_amount_of_days(self):
        if self.month in ('July', 'May', 'March', 'January', 'December', 'October','August'):  #These months have 31 days
            amount_of_days = 31
        elif self.month in ('June', 'April', 'November', 'September'): #These months have 30 days
            amount_of_days = 30
        elif int(self.year) %4 == 0: #A test for whether the year is a leap year or not. This is sufficient logic in the span of dates we are looking at. 
            amount_of_days = 29
        else:
            amount_of_days = 28
        return amount_of_days